' ###################################################################################################
' #                                        JV-ODE Car Demo                                          #
' #                                  Code by Jim Williams (VIP3R)                                   #
' #                              Devious Codeworks - Copyright  2008                               #
' ###################################################################################################

SuperStrict

Framework DevCode.JVODEOpenGL
Import DevCode.JVODE
Import BRL.Random

AppTitle:String="JV-ODE Car Demo - Duo"

Graphics3D 800,600,0,2

Global CMass:Float=200.0			' ### Car Mass
Global WMass:Float=14.0			' ### Wheel Mass

Global WorldERP:Float=1.0		' ### World Error Correction
Global WorldFriction:Float=70.0	' ### World Friction

Global Torque:Float=24.0			' ### Joint Torque
Global SuspensionHS:Float=0.01	' ### Suspension Hardness/Softness (Higher=Softer - Lower=Harder)

Global Force:Float[3]
Global Steer:Float[3]

Global Car:Int[3]
Global CGeom:Int[3]
Global CMesh:TMesh[3]

Global CarStartY:Int=20

Global ControlTimer:Int=0
Global SteeringTimer:Int=0

Global Wheel:Int[3,5]
Global WGeom:Int[3,5]
Global Joint:Int[3,5]

Type ODEGeom
	Field body:Int
	Field geom:Int
	Field mesh:TMesh
End Type

Global ODEGeomList:TList=CreateList()

' ###################################################################################################

' ### Setup ODE

dInitODE()

Global World:Int=dWorldCreate()
Global Space:Int=dHashSpaceCreate(0)
Global ContactGroup:Int=dJointGroupCreate(0)

dWorldSetAutoDisableFlag(World,1)
dWorldSetGravity(World,0,-0.98,0)
dWorldSetERP(World,WorldERP)

dContactSetMode(dContactSlip1)
dContactSetMu(WorldFriction)

Local mass:Int
Local count:Int
Local carID:Int
Local ode:ODEGeom

For carID=1 To 2
	' ### Create Car Body

	ode:ODEGeom=New ODEGeom
	ode.body=dBodyCreate(World)
	Car[carID]=ode.body
	dBodySetPosition(ode.body,-10+(carID*10),CarStartY,0)
	dBodySetRotation(ode.body,0,0,0)
	mass=dMassCreate()
	dMassSetBoxTotal(mass,CMass,3,1,4)
	dBodySetMass(ode.body,mass)
	dMassDestroy(mass)
	ode.geom=dCreateBox(Space,3,1,4)
	CGeom[carID]=ode.geom
	dGeomSetBody(ode.geom,ode.body)
	ode.mesh=CreateCube()
	CMesh[carID]=ode.mesh
	ScaleMesh ode.mesh,1.5,0.5,2
	If carID=1
		EntityColor ode.mesh,40,80,255
		Else
		EntityColor ode.mesh,40,255,80
		End If
	ListAddLast(ODEGeomList,ode:ODEGeom)

	' ### Create Wheels

	For count=1 To 4
		ode:ODEGeom=New ODEGeom
		ode.body=dBodyCreate(World)
		Wheel[carID,count]=ode.body
		dBodySetPosition(ode.body,0,0,0)
		dBodySetRotation(ode.body,0,0,90)
		mass=dMassCreate()
		dMassSetSphereTotal(mass,WMass,0.7)
		dBodySetMass(ode.body,mass)
		dMassDestroy(mass)
		ode.geom=dCreateSphere(Space,0.7)
		WGeom[carID,count]=ode.geom
		dGeomSetBody(ode.geom,ode.body)
		ode.mesh=CreateCylinder()
		ScaleMesh ode.mesh,0.7,0.25,0.7
		EntityColor ode.mesh,255,255,255
		ListAddLast(ODEGeomList,ode:ODEGeom)
	Next

	dBodySetPosition(Wheel[carID,1],-12+(carID*10),CarStartY-0.5,+2)
	dBodySetPosition(Wheel[carID,2],-8+(carID*10),CarStartY-0.5,+2)
	dBodySetPosition(Wheel[carID,3],-12+(carID*10),CarStartY-0.5,-2)
	dBodySetPosition(Wheel[carID,4],-8+(carID*10),CarStartY-0.5,-2)
Next

' ### Create Objects

SeedRnd MilliSecs()

Local spheres:Int
Local capsules:Int

For spheres=1 To 20
	ode:ODEGeom=New ODEGeom
	ode.body=dBodyCreate(World)
	dBodySetPosition(ode.body,Rand(-100,100),30,Rand(-100,100))
	dBodySetRotation(ode.body,0,0,0)
	ode.geom=dCreateSphere(Space,4)
	dGeomSetBody(ode.geom,ode.body)
	ode.mesh=CreateSphere()
	ScaleMesh ode.mesh,4,4,4
	EntityColor ode.mesh,Rand(0,255),Rand(0,255),Rand(0,255)
	EntityShininess ode.mesh,0.7
	ListAddLast(ODEGeomList,ode:ODEGeom)
Next

For capsules=1 To 20
	ode:ODEGeom=New ODEGeom
	ode.body=dBodyCreate(World)
	dBodySetPosition(ode.body,Rand(-100,100),30,Rand(-100,100))
	dBodySetRotation(ode.body,90,0,0)
	ode.geom=dCreateCapsule(Space,1,8)
	dGeomSetBody(ode.geom,ode.body)
	ode.mesh=CreateCapsule()
	ScaleMesh ode.mesh,1,4,1
	EntityColor ode.mesh,Rand(0,255),Rand(0,255),Rand(0,255)
	EntityShininess ode.mesh,0.7
	ListAddLast(ODEGeomList,ode:ODEGeom)
Next

' ### Create Light

Global Light:TLight=CreateLight()
PositionEntity Light,-40,70,-40
LightColor Light,255,255,255
AmbientLight 130,130,130

' ### Create Camera

Global CameraPivot:TPivot=CreatePivot()

Global Camera:TCamera=CreateCamera(CameraPivot)
CameraClsColor Camera,0,0,0
CameraRange Camera,1,1000

' ### Create Plane

dCreatePlane(Space,0,1,0,0)

Global Plane:TMesh=CreatePlane()
EntityAlpha Plane,0.8

EntityTexture Plane,PlaneTexture

' ###################################################################################################

' ### OpenGL Static Object (White)

Global OpenGLObject:TMesh=CreateCube()
ScaleMesh OpenGLObject,1,4,20
PositionEntity OpenGLObject,10,5,40
RotateEntity OpenGLObject,-10,8,4
EntityColor OpenGLObject,255,255,255

' ### ODE Static Object (Blue)

ode:ODEGeom=New ODEGeom
ode.geom=dCreateBox(Space,2,8,40)
dGeomSetPosition(ode.geom,20,5,40)
dGeomSetRotation(ode.geom,-10,8,4)
ode.mesh=CreateCube()
ScaleMesh ode.mesh,1,4,20
EntityColor ode.mesh,0,100,200
ListAddLast(ODEGeomList,ode:ODEGeom)

' ### ODE Static Object (Red)

ode:ODEGeom=New ODEGeom
ode.geom=dCreateBox(Space,18,0.2,40)
dGeomSetPosition(ode.geom,-20,4.9,40)
dGeomSetRotation(ode.geom,-14,0,0)
ode.mesh=CreateCube()
ScaleMesh ode.mesh,9,0.1,20
EntityColor ode.mesh,200,0,100
ListAddLast(ODEGeomList,ode:ODEGeom)

' ###################################################################################################

SetupCars()

Local PTime:Int
Local PhysicsTime:Float

While Not AppTerminate() And Not KeyDown(KEY_ESCAPE)

	UpdateKeys()

	UpdateCars()

	UpdateGeoms()

	PTime=MilliSecs()

	dSpaceCollide(Space,World,ContactGroup)
	dWorldQuickStep(World,0.1)
	dJointGroupEmpty(ContactGroup)

	PhysicsTime=MilliSecs()-PTime

	UpdateCam()

	RenderWorld

	Text 0,0,"JV-ODE Version "+FixFloat(dGetVersion())
	Text 0,15,"Physics Time:"+FixFloat(PhysicsTime)
	Text 0,50,"Force Car 1:"+Force[1]
	Text 0,65,"Force Car 2:"+Force[2]
	Text 0,80,"Torque:"+Torque
	Text 640,0,"A - Accelerate"
	Text 640,15,"Z - Brake/Reverse"
	Text 640,30,"Arrows - Steering"
	Text 640,45,"Space - Reset Car"

	Flip 1

Wend

dJointGroupDestroy(ContactGroup)
dSpaceDestroy(Space)
dWorldDestroy(World)
dCloseODE()

End

' ###################################################################################################

Function SetupCars()

Local count:Int
Local carID:Int

For carID=1 To 2
	For count=1 To 4
		Joint[carID,count]=dJointCreateHinge2(World,0)
		dJointAttach(Joint[carID,count],Car[carID],Wheel[carID,count])
		dJointSetHinge2Anchor(Joint[carID,count],dBodyGetPositionX(Wheel[carID,count]),dBodyGetPositionY(Wheel[carID,count]),dBodyGetPositionZ(Wheel[carID,count]))
		dJointSetHinge2Axis1(Joint[carID,count],0,1,0)
		dJointSetHinge2Axis2(Joint[carID,count],-1,0,0)
		dJointSetHinge2Param(Joint[carID,count],dParamSuspensionERP,0.8)
		dJointSetHinge2Param(Joint[carID,count],dParamSuspensionCFM,SuspensionHS)
		If count>2
			dJointSetHinge2Param(Joint[carID,count],dParamLoStop,0)
			dJointSetHinge2Param(Joint[carID,count],dParamHiStop,0)
			End If
	Next
Next

End Function

' ###################################################################################################

Function UpdateKeys()

Local AIKey:Int

' ### Player Control - Car 1

If KeyDown(KEY_A)=1
	Force[1]=Force[1]+0.08
	If Force[1]>30 Then Force[1]=30
	End If

If KeyDown(KEY_Z)=1
	Force[1]=Force[1]-0.1
	If Force[1]<-10 Then Force[1]=-10
	If Force[1]>0 Then Force[1]=Force[1]*0.97
	End If

If KeyDown(KEY_A)=0 And KeyDown(KEY_Z)=0 Then Force[1]=Force[1]*0.99

If KeyDown(KEY_LEFT)=1 Or KeyDown(KEY_RIGHT)=1
	If KeyDown(KEY_LEFT)=1
		Steer[1]=0.5
		Else
		Steer[1]=-0.5
		End If
	Else
	Steer[1]=0.0
	End If

If KeyHit(KEY_SPACE)=1
	Force[1]=0.0
	dBodySetRotation(Car[1],dGeomGetPitch(CGeom[1]),dGeomGetYaw(CGeom[1]),0)
	End If

' ### Simple AI - Car 2

If ControlTimer>100
	SeedRnd MilliSecs()
	AIKey=Rand(0,100)

	If AIKey<40
		Force[2]=Force[2]+0.8
		If Force[2]>10 Then Force[2]=10
		End If

	If AIKey=>40 Then Force[2]=Force[2]*0.99

	ControlTimer=0
	End If

If SteeringTimer>100
	SeedRnd MilliSecs()
	AIKey=Rand(0,100)

	Steer[2]=0.0
	If AIKey<30 Then Steer[2]=0.5
	If AIKey>70 Then Steer[2]=-0.5

	SteeringTimer=0
	End If

ControlTimer=ControlTimer+1
SteeringTimer=SteeringTimer+1

End Function

' ###################################################################################################

Function UpdateCars()

Local count:Int
Local carID:Int
Local angle:Float

For carID=1 To 2
	For count=1 To 4
		dBodyEnable(Wheel[carID,count])
	Next

	dBodyEnable(Car[carID])

	For count=1 To 4
		dJointSetHinge2Param(Joint[carID,count],dParamVel2,Force[carID])
		dJointSetHinge2Param(Joint[carID,count],dParamFMax2,Torque)
	Next

	For count=1 To 2
		angle=Steer[carID]-dJointGetHinge2Angle1(Joint[carID,count])
		dJointSetHinge2Param(Joint[carID,count],dParamVel,angle)
		dJointSetHinge2Param(Joint[carID,count],dParamFMax,400)
	Next
Next

End Function

' ###################################################################################################

Function UpdateCam()

PositionEntity Camera,0,3,-12

Local PivotX:Float=Cos(EntityYaw(CMesh[1])-90)+EntityX(CMesh[1])
Local PivotY:Float=EntityY(CMesh[1])
Local PivotZ:Float=Sin(EntityYaw(CMesh[1])-90)+EntityZ(CMesh[1])

PositionEntity CameraPivot,PivotX,PivotY,PivotZ

PointEntity CameraPivot,CMesh[1]

End Function

' ###################################################################################################

Function UpdateGeoms()

Local ode:ODEGeom

For ode:ODEGeom=EachIn ODEGeomList
	PositionEntity ode.mesh,dGeomGetPositionX(ode.geom),dGeomGetPositionY(ode.geom),dGeomGetPositionZ(ode.geom)
	RotateEntity ode.mesh,dGeomGetPitch(ode.geom),dGeomGetYaw(ode.geom),dGeomGetRoll(ode.geom)
Next

End Function

' ###################################################################################################
