' ###################################################################################################
' #                                        JV-ODE Car Demo                                          #
' #                                  Code by Jim Williams (VIP3R)                                   #
' #                              Devious Codeworks - Copyright  2008                               #
' ###################################################################################################

SuperStrict

Framework DevCode.JVODEOpenGL
Import DevCode.JVODE
Import BRL.Random

AppTitle:String="JV-ODE Car Demo - Geom Offset - Sphere"

Graphics3D 800,600,0,2

Global CMass:Float=200.0			' ### Car Mass
Global WMass:Float=14.0			' ### Wheel Mass

Global WorldERP:Float=1.0		' ### World Error Correction
Global WorldFriction:Float=70.0	' ### World Friction

Global Torque:Float=24.0			' ### Joint Torque
Global SuspensionHS:Float=0.01	' ### Suspension Hardness/Softness (Higher=Softer - Lower=Harder)

Global Force:Float=0.0
Global Steer:Float=0.0

Global Car:Int
Global CGeom:Int
Global CMesh:TMesh

Global CarStartY:Int=20

Global Wheel:Int[5]
Global WGeom:Int[5]
Global Joint:Int[5]

Type ODEGeom
	Field body:Int
	Field geom:Int
	Field mesh:TMesh
End Type

Global ODEGeomList:TList=CreateList()

' ###################################################################################################

' ### Setup ODE

dInitODE()

Global World:Int=dWorldCreate()
Global Space:Int=dHashSpaceCreate(0)
Global ContactGroup:Int=dJointGroupCreate(0)

dWorldSetAutoDisableFlag(World,1)
dWorldSetGravity(World,0,-0.98,0)
dWorldSetERP(World,WorldERP)

dContactSetMode(dContactSlip1)
dContactSetMu(WorldFriction)

Local mass:Int
Local count:Int
Local ode:ODEGeom

' ### Create Car Body

ode:ODEGeom=New ODEGeom
ode.body=dBodyCreate(World)
Car=ode.body
dBodySetPosition(ode.body,0,CarStartY,0)
dBodySetRotation(ode.body,0,0,0)
mass=dMassCreate()
dMassSetBoxTotal(mass,CMass,3,1,4)
dBodySetMass(ode.body,mass)
dMassDestroy(mass)
ode.geom=dCreateBox(Space,3,1,4)
CGeom=ode.geom
dGeomSetBody(ode.geom,ode.body)
ode.mesh=CreateCube()
CMesh=ode.mesh
ScaleMesh ode.mesh,1.5,0.5,2
EntityColor ode.mesh,40,80,255
ListAddLast(ODEGeomList,ode:ODEGeom)

' ### Create Wheels

For count=1 To 4
	ode:ODEGeom=New ODEGeom
	ode.body=dBodyCreate(World)
	Wheel[count]=ode.body
	dBodySetPosition(ode.body,0,0,0)
	dBodySetRotation(ode.body,0,0,90)
	mass=dMassCreate()
	dMassSetSphereTotal(mass,WMass,0.7)
	dBodySetMass(ode.body,mass)
	dMassDestroy(mass)
	ode.geom=dCreateSphere(Space,0.7)
	WGeom[count]=ode.geom
	dGeomSetBody(ode.geom,ode.body)
	ode.mesh=CreateCylinder()
	ScaleMesh ode.mesh,0.7,0.25,0.7
	EntityColor ode.mesh,255,255,255
	ListAddLast(ODEGeomList,ode:ODEGeom)
Next

dBodySetPosition(Wheel[1],-2,CarStartY-0.5,+2)
dBodySetPosition(Wheel[2],+2,CarStartY-0.5,+2)
dBodySetPosition(Wheel[3],-2,CarStartY-0.5,-2)
dBodySetPosition(Wheel[4],+2,CarStartY-0.5,-2)

' ### Create Light

Global Light:TLight=CreateLight()
PositionEntity Light,-40,70,-40
LightColor Light,255,255,255
AmbientLight 130,130,130

' ### Create Camera

Global CameraPivot:TPivot=CreatePivot()

Global Camera:TCamera=CreateCamera(CameraPivot)
CameraClsColor Camera,0,0,0
CameraRange Camera,1,1000

' ### Create Plane

dCreatePlane(Space,0,1,0,0)

Global Plane:TMesh=CreatePlane()
EntityAlpha Plane,0.8

EntityTexture Plane,PlaneTexture

' ###################################################################################################

SetupCar()

SetupObjects()

Local PTime:Int
Local PhysicsTime:Float

While Not AppTerminate() And Not KeyDown(KEY_ESCAPE)

	UpdateKeys()

	UpdateCar()

	UpdateGeoms()

	PTime=MilliSecs()

	dSpaceCollide(Space,World,ContactGroup)
	dWorldQuickStep(World,0.1)
	dJointGroupEmpty(ContactGroup)

	PhysicsTime=MilliSecs()-PTime

	UpdateCam()

	RenderWorld

	Text 0,0,"JV-ODE Version "+FixFloat(dGetVersion())
	Text 0,15,"Physics Time:"+FixFloat(PhysicsTime)
	Text 0,50,"Force:"+Force
	Text 0,65,"Torque:"+Torque
	Text 640,0,"A - Accelerate"
	Text 640,15,"Z - Brake/Reverse"
	Text 640,30,"Arrows - Steering"
	Text 640,45,"Space - Reset Car"

	Flip 1

Wend

dJointGroupDestroy(ContactGroup)
dSpaceDestroy(Space)
dWorldDestroy(World)
dCloseODE()

End

' ###################################################################################################

Function SetupCar()

Local count:Int

For count=1 To 4
	Joint[count]=dJointCreateHinge2(World,0)
	dJointAttach(Joint[count],Car,Wheel[count])
	dJointSetHinge2Anchor(Joint[count],dBodyGetPositionX(Wheel[count]),dBodyGetPositionY(Wheel[count]),dBodyGetPositionZ(Wheel[count]))
	dJointSetHinge2Axis1(Joint[count],0,1,0)
	dJointSetHinge2Axis2(Joint[count],-1,0,0)
	dJointSetHinge2Param(Joint[count],dParamSuspensionERP,0.8)
	dJointSetHinge2Param(Joint[count],dParamSuspensionCFM,SuspensionHS)
	If count>2
		dJointSetHinge2Param(Joint[count],dParamLoStop,0)
		dJointSetHinge2Param(Joint[count],dParamHiStop,0)
		End If
Next

End Function

' ###################################################################################################

Function SetupObjects()

Local count:Int
Local xp:Int
Local zp:Int

For count=1 To 32
	AddObjects((xp*34)-120,17,(zp*34)+100)

	xp=xp+1
	If xp>7
		xp=0
		zp=zp+1
		End If
Next

End Function

' ###################################################################################################

Function AddObjects(objectxp:Float,objectyp:Float,objectzp:Float)

Local count:Int
Local ode:ODEGeom
Local OffsetBody:Int

ode:ODEGeom=New ODEGeom
ode.body=dBodyCreate(World)
OffsetBody=ode.body
dBodySetPosition(ode.body,objectxp,objectyp,objectzp)
dBodySetRotation(ode.body,0,0,0)
ode.geom=dCreateSphere(Space,8)
dGeomSetBody(ode.geom,ode.body)
ode.mesh=CreateSphere()
ScaleMesh ode.mesh,8,8,8
EntityColor ode.mesh,255,255,255
EntityShininess ode.mesh,0.7
EntityAlpha ode.mesh,0.8
ListAddLast(ODEGeomList,ode:ODEGeom)

Local transxp:Float=-7
Local transyp:Float=0

For count=1 To 3
	ode:ODEGeom=New ODEGeom
	ode.geom=dCreateSphere(Space,4)
	dGeomSetBody(ode.geom,OffsetBody)
	dGeomSetOffsetPosition(ode.geom,transxp,transyp,0)
	ode.mesh=CreateSphere()
	ScaleMesh ode.mesh,4,4,4
	PositionEntity ode.mesh,transxp,transyp,0
	EntityColor ode.mesh,count*10,count*70,255
	EntityShininess ode.mesh,0.7
	ListAddLast(ODEGeomList,ode:ODEGeom)

	If count=2
		transxp=0
		transyp=-7
		Else
		transxp=transxp+14
		End If
Next

End Function

' ###################################################################################################

Function UpdateKeys()

If KeyDown(KEY_A)=1
	Force=Force+0.08
	If Force>30 Then Force=30
	End If

If KeyDown(KEY_Z)=1
	Force=Force-0.1
	If Force<-10 Then Force=-10
	If Force>0 Then Force=Force*0.97
	End If

If KeyDown(KEY_A)=0 And KeyDown(KEY_Z)=0 Then Force=Force*0.99

If KeyDown(KEY_LEFT)=1 Or KeyDown(KEY_RIGHT)=1
	If KeyDown(KEY_LEFT)=1
		Steer=0.5
		Else
		Steer=-0.5
		End If
	Else
	Steer=0.0
	End If

If KeyHit(KEY_SPACE)=1
	Force=0.0
	dBodySetRotation(Car,dGeomGetPitch(CGeom),dGeomGetYaw(CGeom),0)
	End If

End Function

' ###################################################################################################

Function UpdateCar()

Local count:Int
Local angle:Float

For count=1 To 4
	dBodyEnable(Wheel[count])
Next

dBodyEnable(Car)

For count=1 To 4
	dJointSetHinge2Param(Joint[count],dParamVel2,Force)
	dJointSetHinge2Param(Joint[count],dParamFMax2,Torque)
Next

For count=1 To 2
	angle=Steer-dJointGetHinge2Angle1(Joint[count])
	dJointSetHinge2Param(Joint[count],dParamVel,angle)
	dJointSetHinge2Param(Joint[count],dParamFMax,400)
Next

End Function

' ###################################################################################################

Function UpdateCam()

PositionEntity Camera,0,3,-12

Local PivotX:Float=Cos(EntityYaw(CMesh)-90)+EntityX(CMesh)
Local PivotY:Float=EntityY(CMesh)
Local PivotZ:Float=Sin(EntityYaw(CMesh)-90)+EntityZ(CMesh)

PositionEntity CameraPivot,PivotX,PivotY,PivotZ

PointEntity CameraPivot,CMesh

End Function

' ###################################################################################################

Function UpdateGeoms()

Local ode:ODEGeom

For ode:ODEGeom=EachIn ODEGeomList
	PositionEntity ode.mesh,dGeomGetPositionX(ode.geom),dGeomGetPositionY(ode.geom),dGeomGetPositionZ(ode.geom)
	RotateEntity ode.mesh,dGeomGetPitch(ode.geom),dGeomGetYaw(ode.geom),dGeomGetRoll(ode.geom)
Next

End Function

' ###################################################################################################
