' ###################################################################################################
' #                                       JV-ODE Truck Demo                                         #
' #                                  Code by Jim Williams (VIP3R)                                   #
' #                              Devious Codeworks - Copyright  2008                               #
' ###################################################################################################

SuperStrict

Framework DevCode.JVODEOpenGL
Import DevCode.JVODE
Import BRL.Random

AppTitle:String="JV-ODE Car Demo - Truck & Trailer"

Graphics3D 800,600,0,2

Global CMass:Float=400.0			' ### Cab Mass
Global WMass:Float=14.0			' ### Wheel Mass
Global TMass:Float=800.0			' ### Trailer Mass

Global WorldERP:Float=1.0		' ### World Error Correction
Global WorldFriction:Float=80.0	' ### World Friction

Global Torque:Float=48.0			' ### Joint Torque
Global SuspensionHS:Float=0.01	' ### Suspension Hardness/Softness (Higher=Softer - Lower=Harder)

Global Force:Float=0.0
Global Steer:Float=0.0

Global Cab:Int
Global CGeom:Int
Global CMesh:TMesh

Global Trailer:Int
Global TGeom:Int
Global TMesh:TMesh

Global TruckStartY:Int=4

Global Wheel:Int[7]
Global WGeom:Int[7]
Global Joint:Int[7]

Global TWheel:Int[7]
Global TWGeom:Int[7]
Global TJoint:Int[7]

Type ODEGeom
	Field body:Int
	Field geom:Int
	Field mesh:TMesh
End Type

Global ODEGeomList:TList=CreateList()

' ###################################################################################################

' ### Setup ODE

dInitODE()

Global World:Int=dWorldCreate()
Global Space:Int=dHashSpaceCreate(0)
Global ContactGroup:Int=dJointGroupCreate(0)

dWorldSetAutoDisableFlag(World,1)
dWorldSetGravity(World,0,-0.98,0)
dWorldSetERP(World,WorldERP)

dContactSetMode(dContactSlip1)
dContactSetMu(WorldFriction)

Local mass:Int
Local count:Int
Local ode:ODEGeom

' ### Create Cab

ode:ODEGeom=New ODEGeom
ode.body=dBodyCreate(World)
Cab=ode.body
dBodySetPosition(ode.body,0,TruckStartY,0)
dBodySetRotation(ode.body,0,0,0)
mass=dMassCreate()
dMassSetBoxTotal(mass,CMass,3,1,8)
dBodySetMass(ode.body,mass)
dMassDestroy(mass)
ode.geom=dCreateBox(Space,3,1,8)
CGeom=ode.geom
dGeomSetBody(ode.geom,ode.body)
ode.mesh=CreateCube()
CMesh=ode.mesh
ScaleMesh ode.mesh,1.5,0.5,4
EntityColor ode.mesh,40,80,255
ListAddLast(ODEGeomList,ode:ODEGeom)

ode:ODEGeom=New ODEGeom
ode.geom=dCreateBox(Space,3,2,4)
dGeomSetBody(ode.geom,Cab)
dGeomSetOffsetPosition(ode.geom,0,1.5,2)
ode.mesh=CreateCube()
ScaleMesh ode.mesh,1.5,1,2
EntityColor ode.mesh,40,80,255
EntityAlpha ode.mesh,0.7
ListAddLast(ODEGeomList,ode:ODEGeom)

' ### Create Wheels

For count=1 To 6
	ode:ODEGeom=New ODEGeom
	ode.body=dBodyCreate(World)
	Wheel[count]=ode.body
	dBodySetPosition(ode.body,0,0,0)
	dBodySetRotation(ode.body,0,0,90)
	mass=dMassCreate()
	dMassSetSphereTotal(mass,WMass,0.7)
	dBodySetMass(ode.body,mass)
	dMassDestroy(mass)
	ode.geom=dCreateSphere(Space,0.7)
	WGeom[count]=ode.geom
	dGeomSetBody(ode.geom,ode.body)
	ode.mesh=CreateCylinder()
	ScaleMesh ode.mesh,0.7,0.25,0.7
	EntityColor ode.mesh,255,255,255
	ListAddLast(ODEGeomList,ode:ODEGeom)
Next

dBodySetPosition(Wheel[1],-2,TruckStartY-0.5,+3)
dBodySetPosition(Wheel[2],+2,TruckStartY-0.5,+3)
dBodySetPosition(Wheel[3],-2,TruckStartY-0.5,-1.5)
dBodySetPosition(Wheel[4],+2,TruckStartY-0.5,-1.5)
dBodySetPosition(Wheel[5],-2,TruckStartY-0.5,-3)
dBodySetPosition(Wheel[6],+2,TruckStartY-0.5,-3)

' ### Create Trailer

ode:ODEGeom=New ODEGeom
ode.body=dBodyCreate(World)
Trailer=ode.body
dBodySetPosition(ode.body,0,TruckStartY+2,-10.5)
dBodySetRotation(ode.body,0,0,0)
mass=dMassCreate()
dMassSetBoxTotal(mass,TMass,3,3,18)
dBodySetMass(ode.body,mass)
dMassDestroy(mass)
ode.geom=dCreateBox(Space,3,3,18)
TGeom=ode.geom
dGeomSetBody(ode.geom,ode.body)
ode.mesh=CreateCube()
TMesh=ode.mesh
ScaleMesh ode.mesh,1.5,1.5,9
EntityColor ode.mesh,20,40,128
EntityAlpha ode.mesh,0.7
ListAddLast(ODEGeomList,ode:ODEGeom)

' ### Create Trailer Wheels

For count=1 To 6
	ode:ODEGeom=New ODEGeom
	ode.body=dBodyCreate(World)
	TWheel[count]=ode.body
	dBodySetPosition(ode.body,0,0,0)
	dBodySetRotation(ode.body,0,0,90)
	mass=dMassCreate()
	dMassSetSphereTotal(mass,WMass,0.7)
	dBodySetMass(ode.body,mass)
	dMassDestroy(mass)
	ode.geom=dCreateSphere(Space,0.7)
	TWGeom[count]=ode.geom
	dGeomSetBody(ode.geom,ode.body)
	ode.mesh=CreateCylinder()
	ScaleMesh ode.mesh,0.7,0.25,0.7
	EntityColor ode.mesh,255,255,255
	ListAddLast(ODEGeomList,ode:ODEGeom)
Next

dBodySetPosition(TWheel[1],-2,TruckStartY-0.5,-18)
dBodySetPosition(TWheel[2],+2,TruckStartY-0.5,-18)
dBodySetPosition(TWheel[3],-2,TruckStartY-0.5,-16.5)
dBodySetPosition(TWheel[4],+2,TruckStartY-0.5,-16.5)
dBodySetPosition(TWheel[5],-2,TruckStartY-0.5,-15)
dBodySetPosition(TWheel[6],+2,TruckStartY-0.5,-15)

' ### Create Objects

SeedRnd MilliSecs()

Local spheres:Int
Local capsules:Int

For spheres=1 To 20
	ode:ODEGeom=New ODEGeom
	ode.body=dBodyCreate(World)
	dBodySetPosition(ode.body,Rand(-100,100),30,Rand(-100,100))
	dBodySetRotation(ode.body,0,0,0)
	ode.geom=dCreateSphere(Space,4)
	dGeomSetBody(ode.geom,ode.body)
	ode.mesh=CreateSphere()
	ScaleMesh ode.mesh,4,4,4
	EntityColor ode.mesh,Rand(0,255),Rand(0,255),Rand(0,255)
	EntityShininess ode.mesh,0.7
	ListAddLast(ODEGeomList,ode:ODEGeom)
Next

For capsules=1 To 20
	ode:ODEGeom=New ODEGeom
	ode.body=dBodyCreate(World)
	dBodySetPosition(ode.body,Rand(-100,100),30,Rand(-100,100))
	dBodySetRotation(ode.body,90,0,0)
	ode.geom=dCreateCapsule(Space,1,8)
	dGeomSetBody(ode.geom,ode.body)
	ode.mesh=CreateCapsule()
	ScaleMesh ode.mesh,1,4,1
	EntityColor ode.mesh,Rand(0,255),Rand(0,255),Rand(0,255)
	EntityShininess ode.mesh,0.7
	ListAddLast(ODEGeomList,ode:ODEGeom)
Next

' ### Create Light

Global Light:TLight=CreateLight()
PositionEntity Light,-40,70,-40
LightColor Light,255,255,255
AmbientLight 130,130,130

' ### Create Camera

Global CameraPivot:TPivot=CreatePivot()

Global Camera:TCamera=CreateCamera(CameraPivot)
CameraClsColor Camera,0,0,0
CameraRange Camera,1,1000

' ### Create Plane

dCreatePlane(Space,0,1,0,0)

Global Plane:TMesh=CreatePlane()
EntityAlpha Plane,0.8

EntityTexture Plane,PlaneTexture

' ###################################################################################################

' ### OpenGL Static Object (White)

Global OpenGLObject:TMesh=CreateCube()
ScaleMesh OpenGLObject,1,4,20
PositionEntity OpenGLObject,10,5,40
RotateEntity OpenGLObject,-10,8,4
EntityColor OpenGLObject,255,255,255

' ### ODE Static Object (Blue)

ode:ODEGeom=New ODEGeom
ode.geom=dCreateBox(Space,2,8,40)
dGeomSetPosition(ode.geom,20,5,40)
dGeomSetRotation(ode.geom,-10,8,4)
ode.mesh=CreateCube()
ScaleMesh ode.mesh,1,4,20
EntityColor ode.mesh,0,100,200
ListAddLast(ODEGeomList,ode:ODEGeom)

' ### ODE Static Object (Red)

ode:ODEGeom=New ODEGeom
ode.geom=dCreateBox(Space,18,0.2,40)
dGeomSetPosition(ode.geom,-20,4.9,40)
dGeomSetRotation(ode.geom,-14,0,0)
ode.mesh=CreateCube()
ScaleMesh ode.mesh,9,0.1,20
EntityColor ode.mesh,200,0,100
ListAddLast(ODEGeomList,ode:ODEGeom)

' ###################################################################################################

SetupTruck()

Local PTime:Int
Local PhysicsTime:Float

While Not AppTerminate() And Not KeyDown(KEY_ESCAPE)

	UpdateKeys()

	UpdateTruck()

	UpdateGeoms()

	PTime=MilliSecs()

	dSpaceCollide(Space,World,ContactGroup)
	dWorldQuickStep(World,0.1)
	dJointGroupEmpty(ContactGroup)

	PhysicsTime=MilliSecs()-PTime

	UpdateCam()

	RenderWorld

	Text 0,0,"JV-ODE Version "+FixFloat(dGetVersion())
	Text 0,15,"Physics Time:"+FixFloat(PhysicsTime)
	Text 0,50,"Force:"+Force
	Text 0,65,"Torque:"+Torque
	Text 640,0,"A - Accelerate"
	Text 640,15,"Z - Brake/Reverse"
	Text 640,30,"Arrows - Steering"
	Text 640,45,"Space - Reset Truck"

	Flip 1

Wend

dJointGroupDestroy(ContactGroup)
dSpaceDestroy(Space)
dWorldDestroy(World)
dCloseODE()

End

' ###################################################################################################

Function SetupTruck()

Local count:Int
Local AJoint:Int

For count=1 To 6
	Joint[count]=dJointCreateHinge2(World,0)
	dJointAttach(Joint[count],Cab,Wheel[count])
	dJointSetHinge2Anchor(Joint[count],dBodyGetPositionX(Wheel[count]),dBodyGetPositionY(Wheel[count]),dBodyGetPositionZ(Wheel[count]))
	dJointSetHinge2Axis1(Joint[count],0,1,0)
	dJointSetHinge2Axis2(Joint[count],-1,0,0)
	dJointSetHinge2Param(Joint[count],dParamSuspensionERP,0.8)
	dJointSetHinge2Param(Joint[count],dParamSuspensionCFM,SuspensionHS)
	If count>2
		dJointSetHinge2Param(Joint[count],dParamLoStop,0)
		dJointSetHinge2Param(Joint[count],dParamHiStop,0)
		End If
Next

For count=1 To 6
	TJoint[count]=dJointCreateHinge2(World,0)
	dJointAttach(TJoint[count],Trailer,TWheel[count])
	dJointSetHinge2Anchor(TJoint[count],dBodyGetPositionX(TWheel[count]),dBodyGetPositionY(TWheel[count]),dBodyGetPositionZ(TWheel[count]))
	dJointSetHinge2Axis1(TJoint[count],0,1,0)
	dJointSetHinge2Axis2(TJoint[count],-1,0,0)
	dJointSetHinge2Param(TJoint[count],dParamSuspensionERP,0.8)
	dJointSetHinge2Param(TJoint[count],dParamSuspensionCFM,SuspensionHS)
	dJointSetHinge2Param(TJoint[count],dParamLoStop,0)
	dJointSetHinge2Param(TJoint[count],dParamHiStop,0)
Next

AJoint=dJointCreateHinge(World,0)
dJointAttach(AJoint,Trailer,Cab)
dJointSetHingeAnchor(AJoint,dBodyGetPositionX(Cab),dBodyGetPositionY(Cab),dBodyGetPositionZ(Cab)-2)
dJointSetHingeAxis(AJoint,0,1,0)
dJointSetHingeParam(AJoint,dParamSuspensionERP,0.8)
dJointSetHingeParam(AJoint,dParamSuspensionCFM,SuspensionHS)
dJointSetHingeParam(AJoint,dParamLoStop,-2)
dJointSetHingeParam(AJoint,dParamHiStop,2)

End Function

' ###################################################################################################

Function UpdateKeys()

If KeyDown(KEY_A)=1
	Force=Force+0.08
	If Force>30 Then Force=30
	End If

If KeyDown(KEY_Z)=1
	Force=Force-0.1
	If Force<-10 Then Force=-10
	If Force>0 Then Force=Force*0.97
	End If

If KeyDown(KEY_A)=0 And KeyDown(KEY_Z)=0 Then Force=Force*0.99

If KeyDown(KEY_LEFT)=1 Or KeyDown(KEY_RIGHT)=1
	If KeyDown(KEY_LEFT)=1
		Steer=0.5
		Else
		Steer=-0.5
		End If
	Else
	Steer=0.0
	End If

If KeyHit(KEY_SPACE)=1
	Force=0.0
	dBodySetRotation(Cab,dGeomGetPitch(CGeom),dGeomGetYaw(CGeom),0)
	dBodySetRotation(Trailer,dGeomGetPitch(TGeom),dGeomGetYaw(TGeom),0)
	End If

End Function

' ###################################################################################################

Function UpdateTruck()

Local count:Int
Local angle:Float

For count=1 To 6
	dBodyEnable(Wheel[count])
	dBodyEnable(TWheel[count])
Next

dBodyEnable(Cab)
dBodyEnable(Trailer)

For count=1 To 6
	dJointSetHinge2Param(Joint[count],dParamVel2,Force)
	dJointSetHinge2Param(Joint[count],dParamFMax2,Torque)
Next

For count=1 To 2
	angle=Steer-dJointGetHinge2Angle1(Joint[count])
	dJointSetHinge2Param(Joint[count],dParamVel,angle)
	dJointSetHinge2Param(Joint[count],dParamFMax,400)
Next

End Function

' ###################################################################################################

Function UpdateCam()

PositionEntity Camera,0,15,-25
RotateEntity Camera,30,0,0

Local PivotX:Float=Cos(EntityYaw(CMesh)-90)+EntityX(CMesh)
Local PivotY:Float=EntityY(CMesh)
Local PivotZ:Float=Sin(EntityYaw(CMesh)-90)+EntityZ(CMesh)

PositionEntity CameraPivot,PivotX,PivotY,PivotZ

PointEntity CameraPivot,CMesh

End Function

' ###################################################################################################

Function UpdateGeoms()

Local ode:ODEGeom

For ode:ODEGeom=EachIn ODEGeomList
	PositionEntity ode.mesh,dGeomGetPositionX(ode.geom),dGeomGetPositionY(ode.geom),dGeomGetPositionZ(ode.geom)
	RotateEntity ode.mesh,dGeomGetPitch(ode.geom),dGeomGetYaw(ode.geom),dGeomGetRoll(ode.geom)
Next

End Function

' ###################################################################################################
