' ###################################################################################################
' #                                    JV-ODE - 2D Spheres Demo                                     #
' #                                  Code by Jim Williams (VIP3R)                                   #
' #                              Devious Codeworks - Copyright  2008                               #
' ###################################################################################################

SuperStrict

Framework BRL.GLMax2D
Import DevCode.JVODE
Import BRL.Random

AppTitle:String="JV-ODE - 2D Spheres Demo"

SetGraphicsDriver GLMax2DDriver()

Graphics 800,600,0,0

Type ODEGeom
	Field body:Int
	Field geom:Int
	Field rgb:Int[]
End Type

Global ODEGeomList:TList=CreateList()

' ###################################################################################################

' ### Setup ODE

dInitODE()

Global World:Int=dWorldCreate()
Global Space:Int=dHashSpaceCreate(0)
Global ContactGroup:Int=dJointGroupCreate(0)

dWorldSetAutoDisableFlag(World,1)
dWorldSetGravity(World,0,-9.8,0)
dContactSetMode(dContactBounce)
dContactSetBounce(0.8)
dContactSetMu(48)

Local ode:ODEGeom

' ### Create Plane

dCreatePlane(Space,0,1,0,0)

' ###################################################################################################

Local PTime:Int
Local Timer:Int
Local PhysicsTime:Float

While Not AppTerminate() And Not KeyDown(KEY_ESCAPE)

	Cls

	If MilliSecs()-Timer>800
		AddObject()
		Timer=MilliSecs()
		End If

	UpdateGeoms()

	PTime=MilliSecs()

	dSpaceCollide(Space,World,ContactGroup)
	dWorldQuickStep(World,0.05)
	dJointGroupEmpty(ContactGroup)

	PhysicsTime=MilliSecs()-PTime

	DrawText "JV-ODE Version "+FixFloat(dGetVersion()),0,0
	DrawText "Physics Time:"+FixFloat(PhysicsTime),0,15

	Flip 1

Wend

dJointGroupDestroy(ContactGroup)
dSpaceDestroy(Space)
dWorldDestroy(World)
dCloseODE()

End

' ###################################################################################################

Function AddObject()

Local xp:Float
Local ode:ODEGeom

xp=Rand(-80,80)

ode:ODEGeom=New ODEGeom
ode.body=dBodyCreate(World)
dBodySetPosition(ode.body,xp,500,0)
dBodySetRotation(ode.body,0,0,0)
dBodySetAutoDisableFlag(ode.body,1)
ode.geom=dCreateSphere(Space,14)
dGeomSetBody(ode.geom,ode.body)
ode.rgb=[Rand(0,255),Rand(0,255),Rand(0,255)]
ListAddLast(ODEGeomList,ode:ODEGeom)

Local Joint2D:Int=dJointCreatePlane2D(World,0)
dJointAttach(Joint2D,ode.body,0)

End Function

' ###################################################################################################

Function UpdateGeoms()

Local ode:ODEGeom
Local xpos:Float
Local ypos:Float
Local angle:Float
Local px:Float
Local py:Float

For ode:ODEGeom=EachIn ODEGeomList
	If ode.body
		SetColor ode.rgb[0],ode.rgb[1],ode.rgb[2]
		xpos=400+dGeomGetPositionX(ode.geom)
		ypos=580-dGeomGetPositionY(ode.geom)
		DrawOval xpos,ypos,28,28

		SetColor 255,255,255
		angle=-dGeomGetRoll(ode.geom)
		px=Cos(angle)*10+xpos
		py=Sin(angle)*10+ypos
		DrawOval 10+px,10+py,8,8
		End If
Next

End Function

' ###################################################################################################
