' ###################################################################################################
' #                                     JV-ODE - Capsules Demo                                      #
' #                                  Code by Jim Williams (VIP3R)                                   #
' #                              Devious Codeworks - Copyright  2008                               #
' ###################################################################################################

SuperStrict

Framework DevCode.JVODEOpenGL
Import DevCode.JVODE
Import BRL.Random

AppTitle:String="JV-ODE - Capsules Demo"

Graphics3D 800,600,0,2

Type ODEGeom
	Field body:Int
	Field geom:Int
	Field mesh:TMesh
End Type

Global ODEGeomList:TList=CreateList()

' ###################################################################################################

' ### Setup ODE

dInitODE()

Global World:Int=dWorldCreate()
Global Space:Int=dHashSpaceCreate(0)
Global ContactGroup:Int=dJointGroupCreate(0)

dWorldSetAutoDisableFlag(World,1)
dWorldSetGravity(World,0,-0.98,0)
dContactSetMode(dContactBounce)
dContactSetBounce(0.2)
dContactSetMu(48)

' ### Create Light

Global Light:TLight=CreateLight()
PositionEntity Light,-40,70,-40
LightColor Light,255,255,255
AmbientLight 130,130,130

' ### Create Camera

Global Camera:TCamera=CreateCamera()
CameraClsColor Camera,0,0,0
CameraRange Camera,1,1000
PositionEntity Camera,0,20,-50
RotateEntity Camera,30,0,0

' ### Create Plane

dCreatePlane(Space,0,1,0,0)

Global Plane:TMesh=CreatePlane()
EntityAlpha Plane,0.8

EntityTexture Plane,PlaneTexture

' ###################################################################################################

Local PTime:Int
Local Timer:Int
Local PhysicsTime:Float

While Not AppTerminate() And Not KeyDown(KEY_ESCAPE)

	If MilliSecs()-Timer>700
		AddObject()
		Timer=MilliSecs()
		End If

	UpdateGeoms()

	PTime=MilliSecs()

	dSpaceCollide(Space,World,ContactGroup)
	dWorldQuickStep(World,0.1)
	dJointGroupEmpty(ContactGroup)

	PhysicsTime=MilliSecs()-PTime

	RenderWorld

	Text 0,0,"JV-ODE Version "+FixFloat(dGetVersion())
	Text 0,15,"Physics Time:"+FixFloat(PhysicsTime)

	Flip 1

Wend

dJointGroupDestroy(ContactGroup)
dSpaceDestroy(Space)
dWorldDestroy(World)
dCloseODE()

End

' ###################################################################################################

Function AddObject()

Local xp:Float
Local zp:Float
Local ode:ODEGeom

xp=Rand(-10,10)
zp=Rand(-10,10)

ode:ODEGeom=New ODEGeom
ode.body=dBodyCreate(World)
dBodySetPosition(ode.body,xp,30,zp)
dBodySetRotation(ode.body,90,0,0)
dBodySetAutoDisableFlag(ode.body,1)
ode.geom=dCreateCapsule(Space,1,8)
dGeomSetBody(ode.geom,ode.body)
ode.mesh=CreateCapsule()
ScaleMesh ode.mesh,1,4,1
EntityColor ode.mesh,Rand(0,255),Rand(0,255),Rand(0,255)
EntityShininess ode.mesh,0.7
ListAddLast(ODEGeomList,ode:ODEGeom)

End Function

' ###################################################################################################

Function UpdateGeoms()

Local ode:ODEGeom

For ode:ODEGeom=EachIn ODEGeomList
	PositionEntity ode.mesh,dGeomGetPositionX(ode.geom),dGeomGetPositionY(ode.geom),dGeomGetPositionZ(ode.geom)
	RotateEntity ode.mesh,dGeomGetPitch(ode.geom),dGeomGetYaw(ode.geom),dGeomGetRoll(ode.geom)
Next

End Function

' ###################################################################################################
