' ###################################################################################################
' #                                   JV-ODE - Geom Bounce Demo                                     #
' #                                  Code by Jim Williams (VIP3R)                                   #
' #                              Devious Codeworks - Copyright  2008                               #
' ###################################################################################################

SuperStrict

Framework DevCode.JVODEOpenGL
Import DevCode.JVODE
Import BRL.Random

AppTitle:String="JV-ODE - Geom Bounce Demo"

Graphics3D 800,600,0,2

Global Sphere:Int[11]

Type ODEGeom
	Field body:Int
	Field geom:Int
	Field mesh:TMesh
End Type

Global ODEGeomList:TList=CreateList()

' ###################################################################################################

' ### Setup ODE

dInitODE()

Global World:Int=dWorldCreate()
Global Space:Int=dHashSpaceCreate(0)
Global ContactGroup:Int=dJointGroupCreate(0)

dWorldSetAutoDisableFlag(World,1)
dWorldSetGravity(World,0,-0.98,0)
dContactSetMode(dContactBounce)
dContactSetBounce(0.95)
dContactSetMu(48)

' ### Create Light

Global Light:TLight=CreateLight()
PositionEntity Light,-40,70,-40
LightColor Light,255,255,255
AmbientLight 130,130,130

' ### Create Camera

Global Camera:TCamera=CreateCamera()
CameraClsColor Camera,0,0,0
CameraRange Camera,1,1000
PositionEntity Camera,0,40,-30
RotateEntity Camera,30,0,0

' ### Create Plane

dCreatePlane(Space,0,1,0,0)

Global Plane:TMesh=CreatePlane()
EntityAlpha Plane,0.5

EntityTexture Plane,PlaneTexture

' ###################################################################################################

AddObjects()

Local count:Int
Local PTime:Int
Local PhysicsTime:Float

While Not AppTerminate() And Not KeyDown(KEY_ESCAPE)

	UpdateGeoms()

	PTime=MilliSecs()

	dSpaceCollide(Space,World,ContactGroup)
	dWorldQuickStep(World,0.1)
	dJointGroupEmpty(ContactGroup)

	PhysicsTime=MilliSecs()-PTime

	RenderWorld

	Text 0,0,"JV-ODE Version "+FixFloat(dGetVersion())
	Text 0,15,"Physics Time:"+FixFloat(PhysicsTime)
	Text 10,300,"Geom Bounce:"

	For count=1 To 10
		Text 68+(count*58),300,FixFloat(dGeomContactGetBounce(Sphere[count]))
	Next

	Text 10,380,"World Bounce (Plane): "+FixFloat(dContactGetBounce())

	Flip 1

Wend

dJointGroupDestroy(ContactGroup)
dSpaceDestroy(Space)
dWorldDestroy(World)
dCloseODE()

End

' ###################################################################################################

Function AddObjects()

Local mass:Int
Local count:Int
Local ode:ODEGeom

For count=1 To 10
	ode:ODEGeom=New ODEGeom
	ode.body=dBodyCreate(World)
	dBodySetPosition(ode.body,-55+(10*count),45,30)
	dBodySetRotation(ode.body,0,0,0)
	mass=dMassCreate()
	dMassSetSphereTotal(mass,1,4)
	dBodySetMass(ode.body,mass)
	dMassDestroy(mass)
	dBodySetAutoDisableFlag(ode.body,1)
	ode.geom=dCreateSphere(Space,4)
	Sphere[count]=ode.geom
	dGeomContactSetMode(ode.geom,dContactBounce)
	dGeomContactSetBounce(ode.geom,count*0.1)
	dGeomSetBody(ode.geom,ode.body)
	ode.mesh=CreateSphere()
	ScaleMesh ode.mesh,4,4,4
	EntityColor ode.mesh,Rand(0,255),Rand(0,255),Rand(0,255)
	EntityShininess ode.mesh,0.7
	ListAddLast(ODEGeomList,ode:ODEGeom)
Next

End Function

' ###################################################################################################

Function UpdateGeoms()

Local ode:ODEGeom

For ode:ODEGeom=EachIn ODEGeomList
	PositionEntity ode.mesh,dGeomGetPositionX(ode.geom),dGeomGetPositionY(ode.geom),dGeomGetPositionZ(ode.geom)
	RotateEntity ode.mesh,dGeomGetPitch(ode.geom),dGeomGetYaw(ode.geom),dGeomGetRoll(ode.geom)
Next

End Function

' ###################################################################################################
