' ###################################################################################################
' #                                       JV-ODE - Rope Demo                                        #
' #                                  Code by Jim Williams (VIP3R)                                   #
' #                              Devious Codeworks - Copyright  2008                               #
' ###################################################################################################

SuperStrict

Framework DevCode.JVODEOpenGL
Import DevCode.JVODE
Import BRL.Random

AppTitle:String="JV-ODE - Rope Demo"

Graphics3D 800,600,0,2

Global Sphere:Int[11]
Global Joint:Int[11]

Type ODEGeom
	Field body:Int
	Field geom:Int
	Field mesh:TMesh
End Type

Global ODEGeomList:TList=CreateList()

' ###################################################################################################

' ### Setup ODE

dInitODE()

Global World:Int=dWorldCreate()
Global Space:Int=dHashSpaceCreate(0)
Global ContactGroup:Int=dJointGroupCreate(0)

dWorldSetAutoDisableFlag(World,1)
dWorldSetGravity(World,0,-0.98,0)
dContactSetMode(dContactBounce)
dContactSetBounce(0.0)
dContactSetMu(48)

' ### Create Light

Global Light:TLight=CreateLight()
PositionEntity Light,-40,70,-40
LightColor Light,255,255,255
AmbientLight 130,130,130

' ### Create Camera

Global Camera:TCamera=CreateCamera()
CameraClsColor Camera,0,0,0
CameraRange Camera,1,1000
PositionEntity Camera,0,90,-100
RotateEntity Camera,30,0,0

' ### Create Plane

dCreatePlane(Space,0,1,0,0)

Global Plane:TMesh=CreatePlane()
EntityAlpha Plane,0.8

EntityTexture Plane,PlaneTexture

' ###################################################################################################

' ### Create Rope

Local count:Int
Local ode:ODEGeom

For count=1 To 10
	ode:ODEGeom=New ODEGeom
	ode.body=dBodyCreate(World)
	Sphere[count]=ode.body
	dBodySetPosition(ode.body,-30,90-(count*8),0)
	dBodySetRotation(ode.body,0,0,0)
	dBodySetAutoDisableFlag(ode.body,1)
	ode.geom=dCreateSphere(Space,4)
	dGeomSetBody(ode.geom,ode.body)
	ode.mesh=CreateSphere()
	ScaleMesh ode.mesh,4,4,4
	EntityColor ode.mesh,Rand(0,255),Rand(0,255),Rand(0,255)
	EntityShininess ode.mesh,0.7
	ListAddLast(ODEGeomList,ode:ODEGeom)
	Joint[count]=dJointCreateBall(World,0)
	If count=1
		dJointAttach(Joint[count],0,Sphere[count])
		Else
		dJointAttach(Joint[count],Sphere[count-1],Sphere[count])
		End If
	dJointSetBallAnchor(Joint[count],dBodyGetPositionX(Sphere[count]),dBodyGetPositionY(Sphere[count]),dBodyGetPositionZ(Sphere[count]))
Next

' ###################################################################################################

Local PTime:Int
Local Timer:Int
Local PhysicsTime:Float

While Not AppTerminate() And Not KeyDown(KEY_ESCAPE)

	UpdateKeys()

	If MilliSecs()-Timer>700
		AddObject()
		Timer=MilliSecs()
		End If

	UpdateGeoms()

	PTime=MilliSecs()

	dSpaceCollide(Space,World,ContactGroup)
	dWorldQuickStep(World,0.1)
	dJointGroupEmpty(ContactGroup)

	PhysicsTime=MilliSecs()-PTime

	RenderWorld

	Text 0,0,"JV-ODE Version "+FixFloat(dGetVersion())
	Text 0,15,"Physics Time:"+FixFloat(PhysicsTime)
	Text 640,0,"Add Force To Rope"
	Text 640,15,"Using Arrow Keys"

	Flip 1

Wend

dJointGroupDestroy(ContactGroup)
dSpaceDestroy(Space)
dWorldDestroy(World)
dCloseODE()

End

' ###################################################################################################

Function AddObject()

Local ode:ODEGeom

ode:ODEGeom=New ODEGeom
ode.body=dBodyCreate(World)
dBodySetPosition(ode.body,30,80,0)
dBodySetRotation(ode.body,0,0,0)
dBodySetAutoDisableFlag(ode.body,1)
ode.geom=dCreateBox(Space,10,5,10)
dGeomSetBody(ode.geom,ode.body)
ode.mesh=CreateCube()
ScaleMesh ode.mesh,5,2.5,5
EntityColor ode.mesh,Rand(0,255),Rand(0,255),Rand(0,255)
EntityShininess ode.mesh,0.7
ListAddLast(ODEGeomList,ode:ODEGeom)

End Function

' ###################################################################################################

Function UpdateKeys()

If KeyDown(KEY_UP) Then dBodyAddForce(Sphere[10],0,0,10)
If KeyDown(KEY_DOWN) Then dBodyAddForce(Sphere[10],0,0,-10)
If KeyDown(KEY_LEFT) Then dBodyAddForce(Sphere[10],-10,0,0)
If KeyDown(KEY_RIGHT) Then dBodyAddForce(Sphere[10],10,0,0)

End Function

' ###################################################################################################

Function UpdateGeoms()

Local ode:ODEGeom

For ode:ODEGeom=EachIn ODEGeomList
	PositionEntity ode.mesh,dGeomGetPositionX(ode.geom),dGeomGetPositionY(ode.geom),dGeomGetPositionZ(ode.geom)
	RotateEntity ode.mesh,dGeomGetPitch(ode.geom),dGeomGetYaw(ode.geom),dGeomGetRoll(ode.geom)
Next

End Function

' ###################################################################################################
