' ###################################################################################################
' #                                      JV-ODE - Water Demo                                        #
' #                                  Code by Jim Williams (VIP3R)                                   #
' #                              Devious Codeworks - Copyright  2008                               #
' ###################################################################################################

SuperStrict

Framework DevCode.JVODEOpenGL
Import DevCode.JVODE
Import BRL.Random

AppTitle:String="JV-ODE - Water Demo"

Graphics3D 800,600,0,2

Type ODEGeom
	Field body:Int
	Field geom:Int
	Field mesh:TMesh
End Type

Global ODEGeomList:TList=CreateList()

' ###################################################################################################

' ### Setup ODE

dInitODE()

Global World:Int=dWorldCreate()
Global Space:Int=dHashSpaceCreate(0)
Global SceneSpace:Int=dHashSpaceCreate(Space)
Global ContactGroup:Int=dJointGroupCreate(0)

dSetInternalSpaceCollideMode(0)

dWorldSetAutoDisableFlag(World,1)
dWorldSetGravity(World,0,-0.98,0)
dContactSetMode(dContactSlip1)
dContactSetMu(0.5)

' ### Create Light

Global Light:TLight=CreateLight()
PositionEntity Light,-40,70,-40
LightColor Light,255,255,255
AmbientLight 130,130,130

' ### Create Camera

Global Camera:TCamera=CreateCamera()
CameraClsColor Camera,0,0,0
CameraRange Camera,1,1000
PositionEntity Camera,0,45,-65
RotateEntity Camera,30,0,0

' ### Create Plane

dCreatePlane(SceneSpace,0,1,0,0)

Global Plane:TMesh=CreatePlane()
EntityAlpha Plane,0.8

EntityTexture Plane,PlaneTexture

' ###################################################################################################

Local ode:ODEGeom

' ### Create Water

ode:ODEGeom=New ODEGeom
ode.geom=dCreateBox(SceneSpace,100,15,100)
dGeomSetPosition(ode.geom,0,7.5,0)
dGeomSetRotation(ode.geom,0,0,0)
dGeomContactSetMode(ode.geom,dContactBounce+dContactSoftCFM)
dGeomContactSetBounce(ode.geom,0.1)	' ### Water Bounce
dGeomContactSetSoftCFM(ode.geom,0.8)	' ### Water Density
dGeomContactSetMu(ode.geom,0.05)		' ### Water Friction
ode.mesh=CreateCube()
ScaleMesh ode.mesh,50,7.5,50
EntityColor ode.mesh,0,70,200
EntityShininess ode.mesh,0.7
EntityAlpha ode.mesh,0.5
ListAddLast(ODEGeomList,ode:ODEGeom)

' ### Create Barriers

ode:ODEGeom=New ODEGeom
ode.geom=dCreateBox(SceneSpace,100,18,2)
dGeomSetPosition(ode.geom,1,9,-50)
dGeomSetRotation(ode.geom,0,0,0)
ode.mesh=CreateCube()
ScaleMesh ode.mesh,50,9,1
EntityColor ode.mesh,240,240,240
ListAddLast(ODEGeomList,ode:ODEGeom)

ode:ODEGeom=New ODEGeom
ode.geom=dCreateBox(SceneSpace,100,18,2)
dGeomSetPosition(ode.geom,-1,9,50)
dGeomSetRotation(ode.geom,0,0,0)
ode.mesh=CreateCube()
ScaleMesh ode.mesh,50,9,1
EntityColor ode.mesh,240,240,240
ListAddLast(ODEGeomList,ode:ODEGeom)

ode:ODEGeom=New ODEGeom
ode.geom=dCreateBox(SceneSpace,2,18,100)
dGeomSetPosition(ode.geom,-50,9,-1)
dGeomSetRotation(ode.geom,0,0,0)
ode.mesh=CreateCube()
ScaleMesh ode.mesh,1,9,50
EntityColor ode.mesh,240,240,240
ListAddLast(ODEGeomList,ode:ODEGeom)

ode:ODEGeom=New ODEGeom
ode.geom=dCreateBox(SceneSpace,2,18,100)
dGeomSetPosition(ode.geom,50,9,1)
dGeomSetRotation(ode.geom,0,0,0)
ode.mesh=CreateCube()
ScaleMesh ode.mesh,1,9,50
EntityColor ode.mesh,240,240,240
ListAddLast(ODEGeomList,ode:ODEGeom)

' ###################################################################################################

Local PTime:Int
Local Timer:Int
Local PhysicsTime:Float

While Not AppTerminate() And Not KeyDown(KEY_ESCAPE)

	If MilliSecs()-Timer>700
		AddObject()
		Timer=MilliSecs()
		End If

	UpdateGeoms()

	PTime=MilliSecs()

	dSpaceCollide(Space,World,ContactGroup)
	dWorldQuickStep(World,0.1)
	dJointGroupEmpty(ContactGroup)

	PhysicsTime=MilliSecs()-PTime

	RenderWorld

	Text 0,0,"JV-ODE Version "+FixFloat(dGetVersion())
	Text 0,15,"Physics Time:"+FixFloat(PhysicsTime)

	Flip 1

Wend

dJointGroupDestroy(ContactGroup)
dSpaceDestroy(Space)
dWorldDestroy(World)
dCloseODE()

End

' ###################################################################################################

Function AddObject()

Local xp:Float
Local zp:Float
Local mass:Int
Local ode:ODEGeom

xp=Rand(-20,20)
zp=Rand(-20,20)

ode:ODEGeom=New ODEGeom
ode.body=dBodyCreate(World)
dBodySetPosition(ode.body,xp,30,zp)
dBodySetRotation(ode.body,0,0,0)
dBodySetAutoDisableFlag(ode.body,1)
mass=dMassCreate()
dMassSetSphereTotal(mass,15,3)
dBodySetMass(ode.body,mass)
dMassDestroy(mass)
ode.geom=dCreateSphere(Space,3)
dGeomSetBody(ode.geom,ode.body)
ode.mesh=CreateSphere()
ScaleMesh ode.mesh,3,3,3
EntityColor ode.mesh,Rand(0,255),Rand(0,255),Rand(0,255)
EntityShininess ode.mesh,0.7
ListAddLast(ODEGeomList,ode:ODEGeom)

End Function

' ###################################################################################################

Function UpdateGeoms()

Local ode:ODEGeom

For ode:ODEGeom=EachIn ODEGeomList
	PositionEntity ode.mesh,dGeomGetPositionX(ode.geom),dGeomGetPositionY(ode.geom),dGeomGetPositionZ(ode.geom)
	RotateEntity ode.mesh,dGeomGetPitch(ode.geom),dGeomGetYaw(ode.geom),dGeomGetRoll(ode.geom)
Next

End Function

' ###################################################################################################
